import { useEffect, useState } from 'preact/hooks';
import API_BASE_URL from '../../config';
import './style.css';

export function Home() {
    const [groups, setGroups] = useState([]);
    const [selectedGroup, setSelectedGroup] = useState(null);
    const [passwords, setPasswords] = useState([]);
    const [revealedPasswords, setRevealedPasswords] = useState({});
    const [searchTerm, setSearchTerm] = useState('');
    const [copiedStates, setCopiedStates] = useState({});
    const [showModal, setShowModal] = useState(false);
    const [newPasswordData, setNewPasswordData] = useState({ name: '', password: '', group_id: '' });
    const [isPasswordRevealed, setIsPasswordRevealed] = useState(false);

    const signOut = () => {
        localStorage.clear();
        window.location.href = '/login';
    };

    const getAuthToken = () => {
        return localStorage.getItem('accessToken');
    };

    const fetchPasswords = async () => {
        if (selectedGroup) {
            try {
                const response = await fetch(`${API_BASE_URL}/password/${selectedGroup}`, {
                    method: 'GET',
                    headers: {
                        Authorization: `Bearer ${getAuthToken()}`
                    }
                });

                if (response.status === 401) {
                    signOut();
                    return;
                }

                if (!response.ok) {
                    throw new Error(`HTTP error! Status: ${response.status}`);
                }

                const data = await response.json();
                if (Array.isArray(data)) {
                    const sortedPasswords = data.sort((a, b) => a.name.localeCompare(b.name));
                    setPasswords(sortedPasswords);
                } else {
                    console.error('Expected an array, but got:', data);
                    setPasswords([]);
                }
            } catch (error) {
                console.error('Error fetching passwords:', error);
                setPasswords([]);
            }
        }
    };

    useEffect(() => {
        const fetchGroups = async () => {
            try {
                const response = await fetch(`${API_BASE_URL}/password`, {
                    method: 'GET',
                    headers: {
                        Authorization: `Bearer ${getAuthToken()}`
                    }
                });

                if (response.status === 401) {
                    signOut();
                    return;
                }

                if (!response.ok) {
                    throw new Error(`HTTP error! Status: ${response.status}`);
                }

                const data = await response.json();
                if (Array.isArray(data)) {
                    const sortedGroups = data.sort((a, b) => a.GroupName.localeCompare(b.GroupName));
                    setGroups(sortedGroups);
                    if (sortedGroups.length > 0) {
                        setSelectedGroup(sortedGroups[0].GroupID);
                    }
                } else {
                    console.error('Expected an array, but got:', data);
                    setGroups([]);
                }
            } catch (error) {
                console.error('Error fetching groups:', error);
                setGroups([]);
            }
        };

        fetchGroups();
    }, []);

    useEffect(() => {
        fetchPasswords();
    }, [selectedGroup]);

    const toggleRevealPassword = (entryId) => {
        setRevealedPasswords((prev) => ({
            ...prev,
            [entryId]: !prev[entryId]
        }));
    };

    const copyToClipboard = (password, entryId) => {
        navigator.clipboard.writeText(password)
            .then(() => {
                setCopiedStates((prev) => ({ ...prev, [entryId]: true }));
                setTimeout(() => {
                    setCopiedStates((prev) => ({ ...prev, [entryId]: false }));
                }, 2000);
            })
            .catch((err) => {
                console.error('Failed to copy: ', err);
            });
    };

    const filteredPasswords = passwords.filter(entry =>
        entry.name.toLowerCase().includes(searchTerm.toLowerCase())
    );

    const handleCreatePasswordClick = () => {
        setNewPasswordData({ name: '', password: '', group_id: selectedGroup || '' });
        setShowModal(true);
    };

    const generatePassword = () => {
        const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*()';
        let password = '';
        const length = 18;
        for (let i = 0; i < length; i++) {
            password += characters.charAt(Math.floor(Math.random() * characters.length));
        }
        setNewPasswordData((prev) => ({ ...prev, password }));
    };

    const handleCreatePasswordSubmit = async (e) => {
        e.preventDefault();

        try {
            const response = await fetch(`${API_BASE_URL}/password/${newPasswordData.group_id}`, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    Authorization: `Bearer ${getAuthToken()}`
                },
                body: JSON.stringify({
                    name: newPasswordData.name,
                    password: newPasswordData.password
                })
            });

            if (!response.ok) {
                throw new Error('Failed to create password');
            }

            setShowModal(false);
            setNewPasswordData({ name: '', password: '', group_id: '' });
            fetchPasswords();
        } catch (error) {
            console.error('Error creating password:', error);
        }
    };

    const togglePasswordVisibility = () => {
        setIsPasswordRevealed((prev) => !prev);
    };

    return (
        <div className="home">
            <select 
                value={selectedGroup || ''} 
                onChange={(e) => setSelectedGroup(Number(e.target.value))} 
                className="group-dropdown"
            >
                {groups.map((group) => (
                    <option key={group.GroupID} value={group.GroupID}>
                        {group.GroupName}
                    </option>
                ))}
            </select>

            <input 
                type="text" 
                placeholder="Search passwords..." 
                value={searchTerm} 
                onInput={(e) => setSearchTerm(e.target.value)} 
                className="search-input" 
            />

            <button className="btn create-btn" onClick={handleCreatePasswordClick}>
                Create Password
            </button>

            {/* Modal */}
            {showModal && (
                <div className="modal">
                    <div className="modal-content">
                        <h2>Create New Password</h2>
                        <form onSubmit={handleCreatePasswordSubmit}>
                            <select
                                value={newPasswordData.group_id}
                                onChange={(e) => setNewPasswordData({ ...newPasswordData, group_id: e.target.value })}
                                className="group-dropdown"
                                required
                            >
                                <option value="" disabled>Select a Group</option>
                                {groups.map((group) => (
                                    <option key={group.GroupID} value={group.GroupID}>
                                        {group.GroupName}
                                    </option>
                                ))}
                            </select>

                            <input 
                                type="text" 
                                placeholder="Name" 
                                value={newPasswordData.name}
                                onInput={(e) => setNewPasswordData({ ...newPasswordData, name: e.target.value })}
                                required 
                                className="search-input"
                            />

                            <div className="password-input-container">
                                <input 
                                    type={isPasswordRevealed ? "text" : "password"}
                                    placeholder="Password" 
                                    value={newPasswordData.password}
                                    onInput={(e) => setNewPasswordData({ ...newPasswordData, password: e.target.value })}
                                    required 
                                    className="search-input"
                                />
                                <button 
                                    type="button" 
                                    className="btn reveal-btn" 
                                    onClick={togglePasswordVisibility}
                                >
                                    {isPasswordRevealed ? 'Hide' : 'Show'}
                                </button>
                                <button 
                                    type="button" 
                                    className="btn generate-btn" 
                                    onClick={generatePassword}
                                >
                                    Generate
                                </button>
                            </div>

                            <div className="modal-actions">
                                <button type="submit" className="btn submit-btn">
                                    Submit
                                </button>
                                <button type="button" className="btn cancel-btn" onClick={() => setShowModal(false)}>
                                    Cancel
                                </button>
                            </div>
                        </form>
                    </div>
                </div>
            )}

            <section>
                {filteredPasswords.length > 0 ? (
                    filteredPasswords.map((entry) => (
                        <div key={entry.id} className="resource">
                            <h3>{entry.name}</h3>
                            <div className="password-details">
                                <p>
                                    Password: {revealedPasswords[entry.id] ? entry.password : '●●●●●●●●●●'}
                                </p>
                                <button 
                                    className="btn reveal-btn" 
                                    onClick={() => toggleRevealPassword(entry.id)}
                                >
                                    {revealedPasswords[entry.id] ? 'Hide Password' : 'Reveal Password'}
                                </button>
                                <button 
                                    className="btn copy-btn" 
                                    onClick={(e) => {
                                        e.stopPropagation();
                                        copyToClipboard(entry.password, entry.id);
                                    }}
                                >
                                    {copiedStates[entry.id] ? 'Password copied' : 'Copy Password'}
                                </button>
                                <p>Created By: {entry.created_by}</p>
                                <p>Created At: {entry.created_at}</p>
                            </div>
                        </div>
                    ))
                ) : (
                    <p>No passwords available.</p>
                )}
            </section>
        </div>
    );
}
